@@ -1,3 +1,4 @@
+
 ###################################################################
 #  Numexpr - Fast numerical array expression evaluator for NumPy.
 #
@@ -36,12 +37,12 @@ import unittest
 TestCase = unittest.TestCase
 
 double = np.double
-if sys.version_info[0] >= 3:
-    long = int
+long = int
 
 # Recommended minimum versions
-from distutils.version import LooseVersion
-minimum_numpy_version = LooseVersion('1.7.0')
+from packaging.version import Version
+minimum_numpy_version = Version('1.7.0')
+present_numpy_version = Version(np.__version__)
 
 class test_numexpr(TestCase):
     """Testing with 1 thread"""
@@ -378,17 +379,6 @@ class test_evaluate(TestCase):
         x = re_evaluate()
         assert_array_equal(x, array([86., 124., 168.]))
 
-    # Test for issue #37
-    if sys.version_info[0] < 3:
-        # In python 3 '/' perforns true division, not integer division.
-        # Integer division '//' is still not suppoerted by numexpr
-        def test_zero_div(self):
-            x = arange(100, dtype='i4')
-            y = evaluate("1/x")
-            x2 = zeros(100, dtype='i4')
-            x2[1] = 1
-            assert_array_equal(x2, y)
-
     # Test for issue #22
     def test_true_div(self):
         x = arange(10, dtype='i4')
@@ -491,6 +481,14 @@ class test_evaluate(TestCase):
         else:
             self.fail()
 
+    def test_unaligned_singleton(self):
+        # Test for issue #397 whether singletons outputs assigned to consts must be 
+        # aligned or not.
+        a = np.empty(5, dtype=np.uint8)[1:].view(np.int32)
+        evaluate('3', out=a)
+        assert_equal(a, 3)
+
+
     def test_ex_uses_vml(self):
         vml_funcs = [ "sin", "cos", "tan", "arcsin", "arccos", "arctan",
                       "sinh", "cosh", "tanh", "arcsinh", "arccosh", "arctanh",
@@ -505,7 +503,7 @@ class test_evaluate(TestCase):
         # Execution order set here so as to not use too many threads
         # during the rest of the execution.  See #33 for details.
         def test_changing_nthreads_00_inc(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(1, 7):
                 numexpr.set_num_threads(nthreads)
@@ -513,7 +511,7 @@ class test_evaluate(TestCase):
                 assert_array_almost_equal(b, c)
 
         def test_changing_nthreads_01_dec(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(6, 1, -1):
                 numexpr.set_num_threads(nthreads)
@@ -726,13 +724,6 @@ class test_int64(TestCase):
 
 
 class test_int32_int64(TestCase):
-    if sys.version_info[0] < 2:
-        # no long literals in python 3
-        def test_small_long(self):
-            # Small longs should not be downgraded to ints.
-            res = evaluate('42L')
-            assert_array_equal(res, 42)
-            self.assertEqual(res.dtype.name, 'int64')
 
     def test_small_int(self):
         # Small ints (32-bit ones) should not be promoted to longs.
@@ -810,9 +801,14 @@ class test_strings(TestCase):
         str_list = [
             b'\0\0\0', b'\0\0foo\0', b'\0\0foo\0b', b'\0\0foo\0b\0',
             b'foo\0', b'foo\0b', b'foo\0b\0', b'foo\0bar\0baz\0\0']
+        min_tobytes_version = Version('1.9.0')
         for s in str_list:
             r = evaluate('s')
-            self.assertEqual(s, r.tostring())  # check *all* stored data
+            if present_numpy_version >= min_tobytes_version:
+                self.assertEqual(s, r.tobytes())  # check *all* stored data
+            else:
+                # ndarray.tostring() is deprecated as of NumPy 1.19
+                self.assertEqual(s, r.tostring())  # check *all* stored data
 
     def test_compare_copy(self):
         sarr = self.str_array1
@@ -964,10 +960,11 @@ class test_threading_config(TestCase):
         script = '\n'.join([
                 "import os",
                 "if 'NUMEXPR_MAX_THREADS' in os.environ: os.environ.pop('NUMEXPR_MAX_THREADS')",
+                "if 'OMP_NUM_THREADS' in os.environ: os.environ.pop('OMP_NUM_THREADS')",
                 "import numexpr",
                 "assert(numexpr.nthreads <= 8)",
                 "exit(0)"])
-        subprocess.check_call(['python', '-c', script])
+        subprocess.check_call([sys.executable, '-c', script])
 
     def test_max_threads_set(self):
         # Has to be done in a subprocess as `importlib.reload` doesn't let us 
@@ -978,17 +975,33 @@ class test_threading_config(TestCase):
                 "import numexpr",
                 "assert(numexpr.MAX_THREADS == 4)",
                 "exit(0)"])
-        subprocess.check_call(['python', '-c', script])
+        subprocess.check_call([sys.executable, '-c', script])
 
     def test_numexpr_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
             # NUMEXPR_NUM_THREADS has priority
             with _environment('NUMEXPR_NUM_THREADS', '3'):
-                self.assertEqual(3, numexpr._init_num_threads())
+                 if 'sparc' in platform.machine():
+                     self.assertEqual(1, numexpr._init_num_threads())
+                 else:
+                     self.assertEqual(3, numexpr._init_num_threads())
 
     def test_omp_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
-            self.assertEqual(5, numexpr._init_num_threads())
+            if 'sparc' in platform.machine():
+                self.assertEqual(1, numexpr._init_num_threads())
+            else:
+                self.assertEqual(5, numexpr._init_num_threads())
+
+    def test_vml_threads_round_trip(self):
+        n_threads = 3
+        if use_vml:
+            numexpr.utils.set_vml_num_threads(n_threads)
+            set_threads = numexpr.utils.get_vml_num_threads()
+            self.assertEqual(n_threads, set_threads)
+        else:
+            self.assertIsNone(numexpr.utils.set_vml_num_threads(n_threads))
+            self.assertIsNone(numexpr.utils.get_vml_num_threads())
 
 
 # Case test for threads
@@ -1062,7 +1075,7 @@ def print_versions():
     from numexpr.cpuinfo import cpu
     import platform
 
-    np_version = LooseVersion(np.__version__)
+    np_version = Version(np.__version__)
 
     if np_version < minimum_numpy_version:
         print('*Warning*: NumPy version is lower than recommended: %s < %s' % (np_version, minimum_numpy_version))
@@ -1094,7 +1107,6 @@ def test(verbosity=1):
     """
     Run all the tests in the test suite.
     """
-
     print_versions()
     # For some reason, NumPy issues all kinds of warnings when using Python3.
     # Ignoring them in tests should be ok, as all results are checked out.
